;///////////////////////////////////////////////////////////////////////////////////////////////////
;/										  Tic-Tac-Toe					   						   /
;/							   Written by Donald Straney 10/3/04								   /
;/						Designed for the Z80/TI-83+ graphing calculator							   /
;///////////////////////////////////////////////////////////////////////////////////////////////////

#define TI83P
#include "ion.inc"

;CONSTANTS

Pos_1		equ		$1020				;Variable-width cursor positions for board squares
Pos_2		equ		$1029
Pos_3		equ		$1033

Pos_4		equ		$1a20
Pos_5		equ		$1a29
Pos_6		equ		$1a33

Pos_7		equ		$2420
Pos_8		equ		$2429
Pos_9		equ		$2433

Msg_Pos_1	equ		$2e1c				;Variable-width cursor positions for messages
Msg_Pos_2	equ		$340d
Msg_Pos_3	equ		$3a23

Title_Pos_1	equ		$000d				;Variable-width cursor positions for title messages
Title_Pos_2	equ		$0608

Num_X		equ		1					;Numbers on virtual board representing pieces
Num_O		equ		2
Num_Blank	equ		0

Delay_Coeff	equ		65535				;Delay between win, new round

;VARS

Board_Pos_1	equ		savesscreen			;Virtual board-each represents a square on the board
Board_Pos_2	equ		savesscreen+1			;a 0,1, or 2 is stored in each to represent blank, X, or O
Board_Pos_3	equ		savesscreen+2
Board_Pos_4	equ		savesscreen+3
Board_Pos_5	equ		savesscreen+4
Board_Pos_6	equ		savesscreen+5
Board_Pos_7	equ		savesscreen+6
Board_Pos_8	equ		savesscreen+7
Board_Pos_9	equ		savesscreen+8

Go_First	equ		savesscreen+9			;Sets which player goes first-0 is user, 1 is calc		

C_Wins		equ		savesscreen+10			;Number of calc's wins
Y_Wins		equ		savesscreen+11			;Number of user's wins
Ties		equ		savesscreen+12			;Number of ties

Pos_Flags	equ		savesscreen+13			;Second virtual board-1 bit/square
Pos_Flags_2	equ		savesscreen+14			;used to test for wins

Test_Num	equ		savesscreen+15			;Piece to test for on board-used by AI

Save_Num	equ		savesscreen+17


	.org progstart

	ret
	jr nc,New_Game

.db "Tic-Tac-Toe w/AI",0

New_Game:
	bcall(_indicatorOff)				;Make sure that run indicator is off
	ld b,13								;Set to clear all vars
	jr Clear_Vars
New_Round:
	ld b,9								;Set to clear just virtual board-not scores
	ld a,(Go_First)						;Toggle goes-first var
	or a
	jr z,Set_Calc_First					;If user has moved first last round, calc moves first this time
Set_User_First:							;If calc has moved first last round, user moves first this time
	xor a
	jr Set_First_Player
Set_Calc_First:
	ld a,1
Set_First_Player:
	ld (Go_First),a
Clear_Vars:								;Set vars specified in b to 0
	ld hl,savesscreen
	xor a
Clear_Vars_Loop:
	ld (hl),a
	inc hl
	djnz Clear_Vars_Loop
	bcall(_cleargbuf)					;When done, clear screen and text shadow
	call Draw_Text						;Draw all messages, etc.
	ld ix,Board_Pic						;Draw board on screen
	ld a,32
	ld l,15
	ld bc,$1d04
	call ionLargeSprite
;	call ionFastCopy
	call Copy_Screen

	ld a,(Go_First)						;See which player is supposed to go first
	or a
	jp nz,AI_Decide						;If calc goes first, let it move
Key_Loop:								;Keypress loop
	bcall(_getkey)						;Wait for keypress
	sub $09								;If clear key,
	ret z								;then return
	sub $86								;If 1 key,
	jr z,Draw_X_7						;draw an x in lower-left
	dec a								;If 2 key,
	jr z,Draw_X_8						;draw an x in lower-middle
	dec a								;If 3 key,
	jr z,Draw_X_9						;draw an x in lower-right
	dec a								;If 4 key,
	jr z,Draw_X_4						;draw an x in center-left
	dec a								;If 5 key,
	jr z,Draw_X_5						;draw an x in center-middle
	dec a								;If 6 key,
	jr z,Draw_X_6						;draw an x in center-right
	dec a								;If 7 key,
	jr z,Draw_X_1						;draw an x in upper-left
	dec a								;If 8 key,
	jr z,Draw_X_2						;draw an x in upper-middle
	dec a								;If 9 key,
	jr z,Draw_X_3						;draw an x in upper-right
	jr Key_Loop							;If none of these, wait for another keypress

Draw_X_1:
	ld hl,Pos_1							;Point to first position on board
	ld de,Board_Pos_1					;Point to virtual board byte
	jr Draw_X							;Now draw it
Draw_X_2:
	ld hl,Pos_2
	ld de,Board_Pos_2
	jr Draw_X
Draw_X_3:
	ld hl,Pos_3
	ld de,Board_Pos_3
	jr Draw_X
Draw_X_4:
	ld hl,Pos_4
	ld de,Board_Pos_4
	jr Draw_X
Draw_X_5:
	ld hl,Pos_5
	ld de,Board_Pos_5
	jr Draw_X
Draw_X_6:
	ld hl,Pos_6
	ld de,Board_Pos_6
	jr Draw_X
Draw_X_7:
	ld hl,Pos_7
	ld de,Board_Pos_7
	jr Draw_X
Draw_X_8:
	ld hl,Pos_8
	ld de,Board_Pos_8
	jr Draw_X
Draw_X_9:
	ld hl,Pos_9
	ld de,Board_Pos_9	
Draw_X:
	ld a,(de)						;Look at position on virtual board
	or a							;Is it clear?
	jr nz,Key_Loop					;If it isn't, then don't put the X there-wait for another keypress
Draw_X_Now:							;If it is clear, draw an X
	ld a,Num_X
	ld (de),a
	ld a,l
	ld l,h
	ld ix,Sprite_X
	ld b,8
	call ionPutSprite
;	call ionFastCopy
;	call Draw_Text
	call Copy_Screen

	call Test_User_Win				;See if user has 3 Xs in a row
	jr nz,Check_Board_Full			;If user hasn't won, check if board is full, then AI will move
	ld hl,Y_Wins					;If user has won, increment score
	inc (hl)
	call Delay						;wait a second
	jp New_Round					;and start a new round
Check_Board_Full:
	call Test_Board_Full			;Test if board is full
	jr z,AI_Decide					;If it isn't, the AI will move
	ld hl,Ties						;If it is, increment the number of ties
	inc (hl)
	call Delay						;wait a second
	jp New_Round					;then start a new round

AI_Decide:							;Now the AI will decide how to move
	ld a,Num_O						;First, see if can make 3 in a row
	ld (Test_Num),a
	jr Test_Board_Put_O
AI_Try_Block:
	ld a,Num_X						;If not, then try to block user
	ld (Test_Num),a
Test_Board_Put_O:					;Place O to block or to win	
	ld de,Board_Pos_1				;Start at first board position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_1					;If not, skip
	inc de							;If it is, look at position 2
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_3					;If it is, then Xs are on 1,2, put an O on 3 to block
Skip_1:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_2					;If not, skip
	inc de							;If it is, look at position 3
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_2					;If it is, then Xs are on 1,3, put an O on 2 to block
Skip_2:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 2
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_3					;If not, skip
	inc de							;If it is, look at position 3
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_1					;If it is, then Xs are on 2,3, put an O on 1 to block
Skip_3:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_4					;If not, skip
	inc de							;If it is, look at position 4
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_7					;If it is, then Xs are on 1,4, put an O on 7 to block
Skip_4:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_5					;If not, skip
	ld de,Board_Pos_7				;If it is, look at position 7
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_4					;If it is, then Xs are on 1,7, put an O on 4 to block
Skip_5:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 4
	inc de
	inc de
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_6					;If not, skip
	inc de							;If it is, look at position 7
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_1					;If it is, then Xs are on 4,7, put an O on 1 to block
Skip_6:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_7					;If not, skip
	ld de,Board_Pos_5				;If it is, look at position 5
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_9					;If it is, then Xs are on 1,5, put an O on 9 to block
Skip_7:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_8					;If not, skip
	ld de,Board_Pos_9				;If it is, look at position 9
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_5					;If it is, then Xs are on 1,9, put an O on 5 to block
Skip_8:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld de,Board_Pos_5				;Point to space 5
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_9					;If not, skip
	ld de,Board_Pos_9				;If it is, look at position 9
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_1					;If it is, then Xs are on 5,9, put an O on 1 to block
Skip_9:
	ld de,(Save_Num)				;Restore position
	inc de							;Point to space 2
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 5
	inc de
	inc de
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_10					;If not, skip
	inc de							;If it is, look at position 8
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_2					;If it is, then Xs are on 5,8, put an O on 2 to block
Skip_10:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_11					;If not, skip
	inc de							;If it is, look at position 5
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_8					;If it is, then Xs are on 2,5, put an O on 8 to block
Skip_11:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_12					;If not, skip
	ld de,Board_Pos_8				;If it is, look at position 8
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_5					;If it is, then Xs are on 2,8, put an O on 5 to block
Skip_12:
	ld de,(Save_Num)				;Restore position
	inc de							;Point to space 3
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_13					;If not, skip
	inc de							;If it is, look at position 6
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_9					;If it is, then Xs are on 3,6, put an O on 9 to block
Skip_13:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_14					;If not, skip
	ld de,Board_Pos_9				;If it is, look at position 9
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_6					;If it is, then Xs are on 3,9, put an O on 6 to block
Skip_14:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 6
	inc de
	inc de
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_15					;If not, skip
	inc de							;If it is, look at position 9
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_3					;If it is, then Xs are on 6,9, put an O on 3 to block
Skip_15:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_16					;If not, skip
	inc de							;If it is, look at position 5
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_7					;If it is, then Xs are on 3,5, put an O on 7 to block
Skip_16:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_17					;If not, skip
	ld de,Board_Pos_7					;If it is, look at position 7
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_5					;If it is, then Xs are on 3,7, put an O on 5 to block
Skip_17:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 5
	inc de
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_18					;If not, skip
	inc de							;If it is, look at position 7
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_3					;If it is, then Xs are on 5,7, put an O on 3 to block
Skip_18:
	ld de,(Save_Num)				;Restore position
	inc de							;Point to space 4
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_19					;If not, skip
	inc de							;If it is, look at position 5
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_6					;If it is, then Xs are on 4,5, put an O on 6 to block
Skip_19:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_20					;If not, skip
	inc de							;If it is, look at position 6
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_5					;If it is, then Xs are on 4,6, put an O on 5 to block
Skip_20:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 5
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_21					;If not, skip
	inc de							;If it is, look at position 6
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_4					;If it is, then Xs are on 5,6, put an O on 4 to block
Skip_21:
	ld de,(Save_Num)				;Restore position
	inc de							;Point to space 7
	inc de
	inc de
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_22					;If not, skip
	inc de							;If it is, look at position 8
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_9					;If it is, then Xs are on 7,8, put an O on 9 to block
Skip_22:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_23					;If not, skip
	inc de							;If it is, look at position 9
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_8					;If it is, then Xs are on 7,9, put an O on 8 to block
Skip_23:
	ld de,(Save_Num)				;Restore position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 8
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_24					;If not, skip
	inc de							;If it is, look at position 9
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_7					;If it is, then Xs are on 8,9, put an O on 7 to block
Skip_24:
	ld de,(Save_Num)				;Restore position
	ld a,(Test_Num)					;Is it checking for wins?
	cp Num_O
	jp z,AI_Try_Block				;If it is, then hasn't found any possibilities, now try to block user
	jp Put_Random					;Otherwise, put on random square

Draw_O_1:
	ld bc,Board_Pos_1				;Set pointer to virtual board position
	ld hl,Pos_1						;Set cursor position
	jr Put_O						;Check if space is empty, then draw
Draw_O_2:
	ld bc,Board_Pos_2
	ld hl,Pos_2
	jr Put_O
Draw_O_3:
	ld bc,Board_Pos_3
	ld hl,Pos_3
	jr Put_O
Draw_O_4:
	ld bc,Board_Pos_4
	ld hl,Pos_4
	jr Put_O
Draw_O_5:
	ld bc,Board_Pos_5
	ld hl,Pos_5
	jr Put_O
Draw_O_6:
	ld bc,Board_Pos_6
	ld hl,Pos_6
	jr Put_O
Draw_O_7:
	ld bc,Board_Pos_7
	ld hl,Pos_7
	jr Put_O
Draw_O_8:
	ld bc,Board_Pos_8
	ld hl,Pos_8
	jr Put_O
Draw_O_9:
	ld bc,Board_Pos_9
	ld hl,Pos_9
Put_O:
	push hl
	pop de
	pop hl								;Get address that was called from
	ld a,(bc)							;Look at position on board where O would be put
	or a								;Is it empty?
	jr z,Put_O_Now						;If it is, put O on virtual board and screen
	jp (hl)								;Otherwise, return to calling address and check next space
Put_O_Now:
	ld a,Num_O							;Put O on virtual board
	ld (bc),a
	ld l,d
	ld a,e
	ld ix,Sprite_O
	ld b,8
	call ionPutSprite
;	call ionFastCopy
;	call Draw_Text
	call Copy_Screen

	call Test_Calc_Win					;Test if calc has won
	jr nz,Check_Board_Full_2			;If it hasn't, check if board is full, then wait for user to move
	ld hl,C_Wins						;If it has, increment calc's number of wins
	inc (hl)
	call Delay							;wait a second
	jp New_Round						;and start new round
Check_Board_Full_2:
	call Test_Board_Full				;Test if board is full
	jp z,Key_Loop						;If it isn't, the user will move
	ld hl,Ties							;If it is, increment the number of ties
	inc (hl)	
	call Delay							;wait a second
	jp New_Round						;then start a new round

Put_Random:
	call ionRandom						;Generate a random number
	and $0f								;Get the lower 4 bits-must be 15 or under
	sub 9								;Is above 8?
	jr nc,Jump_To_Random				;If it isn't, go to space defined by r
	add a,9								;Otherwise, add 9 again first
Jump_To_Random:
	call z,Draw_O_1						;If 0, go to first space
	dec a
	call z,Draw_O_2						;If 1, go to second space
	dec a
	call z,Draw_O_3						;If 2, go to third space
	dec a
	call z,Draw_O_4						;If 3, go to fourth space
	dec a
	call z,Draw_O_5						;If 4, go to fifth space
	dec a
	call z,Draw_O_6						;If 5, go to sixth space
	dec a
	call z,Draw_O_7						;If 6, go to seventh space
	dec a
	call z,Draw_O_8						;If 7, go to eighth space
	dec a
	call z,Draw_O_9						;If 8, go to ninth space
	jr Put_Random						;If one is taken, then try again

Test_Board_Full:						;Reset z if board is full
	ld hl,Board_Pos_1					;Start at beginning of board
	ld b,9								;Board is 9 squares
Test_Board_Full_Loop:
	ld a,(hl)							;Look at square on board
	or a								;Is there anything there?
	ret z								;If there's a space without anything on it, signal that and return
	inc hl								;Otherwise, look at next space
	djnz Test_Board_Full_Loop			;and loop
	ret									;Only gets here if board is full-last square couldn't have
										;been empty, had to have been 1 or 2, so z is already reset

Test_User_Win:							;Test if user has won
	ld b,Num_X							;Set to recognize user's pieces
	jr Test_Win							;Now test
Test_Calc_Win:							;Test if calc has won
	ld b,Num_O							;Set to recognize calc's pieces
Test_Win:								;Set z if selected player has won has won
										;First, create 9-bit virtual board-each bit corresponds to one
										;square on real board, is set if X is on it (when testing for
										;user win) or if O is on it (testing for calc win), then masks
										;applied to test certain squares for patterns, to detect wins
	ld de,Pos_Flags						;Set pointer to byte w/flags
	ld hl,Board_Pos_1					;Start at beginning of virtual board
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_1						;If not, reset corresponding flag
	call z,Set_1						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_2						;If not, reset corresponding flag
	call z,Set_2						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_3						;If not, reset corresponding flag
	call z,Set_3						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_4						;If not, reset corresponding flag
	call z,Set_4						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_5						;If not, reset corresponding flag
	call z,Set_5						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_6						;If not, reset corresponding flag
	call z,Set_6						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_7						;If not, reset corresponding flag
	call z,Set_7						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_8						;If not, reset corresponding flag
	call z,Set_8						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_9						;If not, reset corresponding flag
	call z,Set_9						;If it is, set corresponding flag
										;Now test flag byte for winning combinations
	ld a,(de)							;Get flag byte 1
	ld b,a								;Save in b
	and %00000111						;Test first combination-mask out all irrelevant bits
	cp %0000111							;Now test
	jr z,End_Test_Positions				;If it matches, then end, z bit is set
	ld a,b								;Restore flag byte
	and %01001001
	cp %01001001
	jr z,End_Test_Positions
	ld a,b
	and %10010010
	cp %10010010
	jr z,End_Test_Positions
	ld a,b
	and %00111000
	cp %00111000
	jr z,End_Test_Positions
	ld a,b
	and %01010100
	cp %01010100
	jr z,End_Test_Positions
	ld a,b
	and %11000000
	cp %11000000
	call z,Test_Bit_9
	ld a,b
	and %00100100
	cp %00100100
	call z,Test_Bit_9
	ld a,b
	and %00010001
	cp %00010001
	call z,Test_Bit_9
End_Test_Positions:						;No wins found-reset z bit
	pop hl								;Get called-from address
	jp (hl)								;Return to that

Test_Bit_9:
	pop hl
	ld a,(Pos_Flags_2)
	cp %00000001
	jr z,End_Test_Positions
	jp (hl)

Set_1:
	ld a,(de)
	set 0,a
	ld (de),a
	ret

Set_2:
	ld a,(de)
	set 1,a
	ld (de),a
	ret

Set_3:
	ld a,(de)
	set 2,a
	ld (de),a
	ret

Set_4:
	ld a,(de)
	set 3,a
	ld (de),a
	ret

Set_5:
	ld a,(de)
	set 4,a
	ld (de),a
	ret

Set_6:
	ld a,(de)
	set 5,a
	ld (de),a
	ret

Set_7:
	ld a,(de)
	set 6,a
	ld (de),a
	ret

Set_8:
	ld a,(de)
	set 7,a
	ld (de),a
	ret

Set_9:
	inc de
	ld a,(de)
	set 0,a
	ld (de),a
	dec de
	ret

Reset_1:
	ld a,(de)
	res 0,a
	ld (de),a
	ret

Reset_2:
	ld a,(de)
	res 1,a
	ld (de),a
	ret

Reset_3:
	ld a,(de)
	res 2,a
	ld (de),a
	ret

Reset_4:
	ld a,(de)
	res 3,a
	ld (de),a
	ret

Reset_5:
	ld a,(de)
	res 4,a
	ld (de),a
	ret

Reset_6:
	ld a,(de)
	res 5,a
	ld (de),a
	ret

Reset_7:
	ld a,(de)
	res 6,a
	ld (de),a
	ret

Reset_8:
	ld a,(de)
	res 7,a
	ld (de),a
	ret

Reset_9:
	inc de
	ld a,(de)
	res 0,a
	ld (de),a
	dec de
	ret

Delay:
	push hl
	pop de
	ld bc,Delay_Coeff
	ldir
	push hl
	pop de
	ld bc,Delay_Coeff
	ldir
	ret

Copy_Screen:
	di
	ld a,$8f
	out ($10),a
	push af
	call LCD_Delay
	ld a,$20
	out ($10),a
	call LCD_Delay
	ld a,$07
	out ($10),a
	call LCD_Delay
	ld hl,gbuf+(15*12)
	ld d,29
Copy_Screen_Loop:
	ld b,12
Copy_Screen_Inner_Loop:
	ld a,(hl)
	out ($11),a
	call LCD_Delay
	inc hl
	djnz Copy_Screen_Inner_Loop
	ld a,$20
	out ($10),a
	call LCD_Delay
	pop af
	inc a
	out ($10),a
	call LCD_Delay
	push af
	dec d
	jr nz,Copy_Screen_Loop
	pop af
	ei
	ret

LCD_Delay:
	push hl
	inc hl
	dec hl
	pop hl
	ret

Draw_Text:
	ld hl,Title_Pos_1					;Set small cursor to top of screen
	ld (_penCol),hl
	ld hl,Title_Text_1					;and draw title text
	bcall(_vputs)
	ld hl,Title_Pos_2
	ld (_penCol),hl
	ld hl,Title_Text_2
	bcall(_vputs)
	ld hl,Msg_Pos_1						;Refresh scores-first set cursor position
	ld (_penCol),hl
	ld hl,Msg_Text_1					;then draw text
	bcall(_vputs)
	ld a,(Y_Wins)						;Get user's score
	call Draw_Score						;and draw
	ld hl,Msg_Pos_2						;Set cursor position for yet another message
	ld (_penCol),hl
	ld hl,Msg_Text_2					;then draw message text again
	bcall(_vputs)
	ld a,(C_Wins)						;Get calc's score
	call Draw_Score						;and draw
	ld hl,Msg_Pos_3						;Set cursor position for last message FINALLY
	ld (_penCol),hl
	ld hl,Msg_Text_3					;then draw LAST MESSAGE!!!
	bcall(_vputs)
	ld a,(Ties)							;Get number of ties
;	call Draw_Score						;and draw!!!

Draw_Score:
	ld l,a								;Put a into hl
	ld h,0
										;Now go on to DisplayHL

;====================================================================
; DisplayHL:  (idea from SCaBBy/Mardell)    [Assembly Coder's Zenith]
;  Display HL as a 5-digit decimal number with no leading zeros
;  out: AF, HL, BC, DE destroyed
;====================================================================
DisplayHL:
 ld c,'0'  				; save ascii value for zero
 ld de,_OP1+5			; point to end of the buffer
 xor a				; zero terminate string
 ld (de),a				; set last byte to zero
DisplayHLl:
 bcall(_divhlby10)		; next digit
 dec de				; next position
 add a,c				; convert to ascii
 ld (de),a				; save digit
 ld a,h				; load upper byte
 or l					; check with lower byte for zero
 jr nz,DisplayHLl			; loop
 ex de,hl				; point to buffer
 bcall(_vputs)			; print number, we're done
 ret

;////////////////////////////////////////////STRINGS////////////////////////////////////////////////

Sprite_X:
;	.db 8, 8
	.db %00000000
	.db %01000100
	.db %00101000
	.db %00010000
	.db %00101000
	.db %01000100
	.db %00000000
	.db %00000000

Sprite_O:
;	.db 8, 8
	.db %00011000
	.db %00100100
	.db %01000010
	.db %01000010
	.db %01000010
	.db %00100100
	.db %00011000
	.db %00000000

Board_Pic:
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %11111111, %11111111, %11111111, %11000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %11111111, %11111111, %11111111, %11000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000

Msg_Text_1:
	.db "Your wins: ",0
Msg_Text_2:
	.db "Calculator's wins:  ",0
Msg_Text_3:
	.db "Ties: ",0

Title_Text_1:
	.db "Tic-Tac-Toe with AI",0
Title_Text_2:
	.db "straney@umbi.umd.edu",0

.end